class Photos

  # ========================================================================= #
  #   RENDER                                                                  #
  # ========================================================================= #

  index: ->
    @gallery()    # Change to camera
    wd.ui.showTab "photos"

  camera: ->
    Ti.Media.showCamera
      saveToPhotoGallery: true
      mediaTypes: [Ti.Media.MEDIA_TYPE_PHOTO]
      animated: false
    
      success: (e) =>
        @create
          media: @process(e.media)
          _s: (photo) ->
            wd.photos.push()
            wd.ui.showTab "home"
            
      cancel: ->
        wd.ui.showTab "home"
    
      error: (error) =>
        if error.code == Ti.Media.NO_CAMERA
          wd.info "No camera detected"
          @gallery()
        else
          alert "Camera error: #{error.code}"

  gallery: ->
    wd.info "Opening gallery"
    Ti.Media.openPhotoGallery
  
      success: (e) =>
        if e.mediaType == Ti.Media.MEDIA_TYPE_PHOTO
          @create
            media: @process(e.media)
            _s: (photo) ->
              wd.photos.push()
              wd.ui.showTab "home"
        else
          alert "Please select a photo"

      cancel: ->
        wd.ui.showTab "home"


  # ========================================================================= #
  #   ACTIONS                                                                 #
  # ========================================================================= #

  create: (args) ->
    photoID = wd.getint("nextPhotoID")
    taleID = wd.getint("currentTale")
    filename = "p#{wd.rjust(photoID, 6)}.jpg"
    
    photo = wd.Photo.create _.extend args,
      filename: filename
      tale_id: taleID
      remote_tale_id: wd.Tale.find(taleID).remote_id
      
    wd.info "Writing photo #{filename}"
    if photo
      wd.setint("nextPhotoID", photoID + 1)
      f = Ti.Filesystem.getFile(Ti.Filesystem.applicationDataDirectory, filename)
      f.write(args.media)
      f = null
      wd.info "Photo #{filename} written"
      
      @addNewPhoto(photo)
      args._s() if args._s


  addNewPhoto: (photo) ->
    if wd.getint("currentTale") == photo.tale_id
      wd.info "Loading new photo #{photo.filename} to homeWebView..."
      wd.ui.homeWebView.evalJS "wd.addNewPhoto('#{photo.filename}')"
      
    if wd.ui.taleShowWin && wd.shownTale && wd.shownTale.id == photo.tale_id
      wd.info "Loading new photo #{photo.filename} to taleWebView..."
      wd.ui.taleWebView.evalJS "wd.addNewPhoto('#{photo.filename}')"


  pull: (tale) ->
    # must sync tales before pulling photos
    wd.info "Pulling photos for #{tale.remote_id}"
    last_pull = tale.last_pull
    tale.update
      last_pull: Math.round(+new Date()/1000)
      
    wd.net.get "api/photos/pull.json?since=#{last_pull}&tale=#{tale.remote_id}",
      _s: (xhr) =>
        wd.info "Pulled #{_.size JSON.parse(xhr.responseText)} photos for #{tale.remote_id}"
        
        photo_remote_ids = _(wd.Photo.all()).pluck("remote_id")
        
        wd.db.ts = Ti.Database.open(wd.db_name)
        wd.db.tsOpen = true
        wd.db.ts.execute "BEGIN IMMEDIATE TRANSACTION"
        for row in JSON.parse(xhr.responseText)
          unless _(photo_remote_ids).include row.id
            wd.info "Getting photo with remote_id #{row.id}"
            params =
              remote_id: row.id
              remote_tale_id: row.tale_id
              user_id: row.user_id
              created_at: row.created_at
              updated_at: row.updated_at
              synced: 1
            
            wd.net.get row.image.url.slice(1),  # Remove leading '/'
              _s: (xhr, params) =>
                wd.info "Creating photo with remote_id #{params.remote_id}"
                @create _.extend params,
                  media: xhr.responseData

              _f: (xhr) ->
                wd.info "Error getting image: #{xhr.responseText}"
            
              , params
              
          # else update
        wd.db.ts.execute "COMMIT TRANSACTION"
        wd.db.tsOpen = false
        wd.db.ts.close()
    
      _f: ->
        wd.debug "Error pulling photos!"
        

  push: ->
    # Must push tales before photos
    unsynced_photos = _.map wd.Photo.where("synced = ?", 0), (photo) ->
      wd.Steel.filter(photo, ["id", "filename", "remote_tale_id", "created_at", "updated_at"])

    wd.info "Pushing #{_.size unsynced_photos} photos"  
  
    for photo in unsynced_photos
      f = Ti.Filesystem.getFile(Ti.Filesystem.applicationDataDirectory, photo.filename)
      media = f.read()
      params = 
        "photo[id]" : photo.id
        "photo[tale_id]": photo.remote_tale_id
        "photo[created_at]": photo.created_at
        "photo[updated_at]": photo.updated_at
        "photo[image]": media
        "user[id]": wd.getint("userID")

      wd.net.post "api/photos/push.json", params, 
        _s: (xhr) ->
          response = JSON.parse(xhr.responseText)
          if _.has(response, "remote_id")
            wd.Photo.find(response.id).update
              synced: 1
              remote_id: response.remote_id
          else
            errorMsg = response.errors.join(", ")
            alert errorMsg
          # Refresh tales to update synced icon?
      
        _f: ->
          wd.debug "Error pushing photos!"

        onsendstream: (e) ->
          wd.info "send stream: #{e.progress}"
      
        media = null

  process: (media) ->
    return @resize @crop media

  crop: (media) ->
    if wd.iphone
      width = media.width
      height = media.height
    else
      preImg = Ti.UI.createImageView
        image: media
      width = preImg.toBlob().width
      height = preImg.toBlob().height

    img = Ti.UI.createImageView
      image: media
      width: width
      height: height

    length = if height < width then height else width
    
    cropped = Ti.UI.createView
      top: 0
      left: 0
      width: length
      height: length
    cropped.add img
    img.left = -(width - length)/2
    img.top = -(height - length)/2

    if wd.android
      postImg = Ti.UI.createImageView
        image: cropped.toImage()
      return postImg.toBlob()
    return cropped.toImage()
    
  resize: (media) ->
    resized = Ti.UI.createImageView
      image: media
      height: 320
      width: 320
    if wd.iphone
      return resized.toImage()
    else
      return resized.toBlob()

wd.photos = new Photos

    
