class Tales

  # ========================================================================= #
  #   RENDER                                                                  #
  # ========================================================================= #
  
  index: =>
    wd.ui.showTab "tales"
    @push()
    @pull()
        
  new: ->
    wd.ui.addTaleWin = wd.ui.createAddTaleWindow()
    wd.ui.openWin(wd.ui.addTaleWin, wd.ui.talesTab)
    
  show: (tale) ->
    wd.shownTale = tale
    wd.ui.navTitle.text = tale.title if wd.android
    
    wd.ui.taleShowWin = wd.ui.createTaleShowWindow()
    wd.ui.openWin(wd.ui.taleShowWin, wd.ui.talesTab)
    
    wd.photos.push()
    wd.photos.pull(tale)


  # ========================================================================= #
  #   ACTIONS                                                                 #
  # ========================================================================= #
    
  create: (params) ->
    tale = wd.Tale.create(params)
    if tale
      @push()
      @pull()
      wd.setint("currentTale", + wd.Tale.last().id)
      wd.ui.addTaleWin.close()

  tales: ->  
    tales = wd.Tale.all() 
    for tale in tales
      tale.title = "#{tale.id} - #{tale.remote_id} - #{tale.title}"
      unless tale["synced"]
        tale.title = "#{tale.title}*"
        tale.color = "#ccc"
    return tales
  
  pull: ->
    wd.info "Pulling tales"
    lastPull = wd.getstr("talesLastPull")
    user = wd.getint("userID")
    wd.net.get "api/tales/pull.json?since=#{lastPull}&user=#{user}",
      _s: (xhr) ->
        wd.info "Pulled #{_.size JSON.parse(xhr.responseText)} tales"

        wd.setstr("talesLastPull", Math.round(+new Date()/1000))
        tale_remote_ids = _(wd.Tale.all()).pluck("remote_id")
        
        wd.db.ts = Ti.Database.open(wd.db_name)
        wd.db.tsOpen = true
        wd.db.ts.execute "BEGIN IMMEDIATE TRANSACTION"
        for row in JSON.parse(xhr.responseText)
          unless _(tale_remote_ids).include row.id
            wd.Tale.create
              remote_id: row.id
              title: row.title
              created_at: row.created_at
              updated_at: row.updated_at
              synced: 1
          else
            wd.Tale.where("remote_id = ?", row.id)[0].update
              title: row.title
              created_at: row.created_at
              updated_at: row.updated_at
            
        wd.db.ts.execute "COMMIT TRANSACTION"
        wd.db.tsOpen = false
        wd.db.ts.close()
        wd.ui.talesTable.refresh()
      
      _f: (xhr) ->
        wd.debug "Error pulling! #{wd.errMsg(xhr)}"


  push: ->
    unsynced_tales = _.map wd.Tale.where("synced = ?", 0), (tale) ->
      wd.Steel.filter(tale, ["id", "title", "created_at", "updated_at"])

    wd.info "Pushing #{_.size unsynced_tales} tales"

    if unsynced_tales.length > 0
      params =
        tale: unsynced_tales
        user:
          id: wd.getint("userID")
    
      wd.net.post "api/tales/push.json", params, 
        _s: (xhr) ->
          for id, val of JSON.parse(xhr.responseText)
            if _.has(val, "remote_id")
              wd.Tale.find(id).update
                synced: 1
                remote_id: val.remote_id
            else
              errorMsg = val.errors.join(", ")
              alert errorMsg          
        
          # UPDATE photos WHERE tale_id = id SET remote_tale_id = remote_id
        
        _f: ->
          wd.debug "Error pushing!"
        
      wd.ui.talesTable.refresh()

  
wd.tales = new Tales

