wd.db = {}

bootstrap = ->
  db = Ti.Database.open(wd.db_name)
  db.execute "DROP TABLE IF EXISTS tales"
  db.execute "DROP TABLE IF EXISTS collabs"
  db.execute "DROP TABLE IF EXISTS photos"
  
  db.execute "CREATE TABLE IF NOT EXISTS tales (
                id INTEGER PRIMARY KEY,
                remote_id INTEGER,
                title TEXT,
                synced INTEGER DEFAULT 0,
                last_pull INTEGER,
                created_at DATETIME DEFAULT CURRENT_TIMESTAMP,
                updated_at DATETIME DEFAULT CURRENT_TIMESTAMP
              )" #needs user_id
  db.execute "CREATE TABLE IF NOT EXISTS collabs (
                id INTEGER PRIMARY KEY,
                remote_id INTEGER,
                collab_tale_id INTEGER,
                collaborator_id INTEGER,
                created_at DATETIME DEFAULT CURRENT_TIMESTAMP,
                updated_at DATETIME DEFAULT CURRENT_TIMESTAMP
              )"
  db.execute "CREATE TABLE IF NOT EXISTS photos (
                id INTEGER PRIMARY KEY,
                remote_id INTEGER,
                synced INTEGER DEFAULT 0,
                tale_id INTEGER,
                user_id INTEGER,
                remote_tale_id INTEGER,
                filename TEXT,
                url TEXT,
                created_at DATETIME DEFAULT CURRENT_TIMESTAMP,
                updated_at DATETIME DEFAULT CURRENT_TIMESTAMP
              )"
  db.close()


resetProperties = ->
  # wd.clear "userID"
  # wd.clear "token"

  wd.clear "talesLastPull"
  wd.setint("currentTale", 1)
  
  wd.setint("nextPhotoID", 1)


wipeDirectory = ->
  no_file = false
  i = 1
  until no_file
    filename = "p#{wd.rjust(i, 6)}.jpg"
    f = Ti.Filesystem.getFile(Ti.Filesystem.applicationDataDirectory, filename)
    if f.exists()
      f.deleteFile()
      i += 1
    else
      no_file = true


copyFile = (path, file) ->
  oldpath = Ti.Filesystem.resourcesDirectory + path.join(Ti.Filesystem.separator)
  oldfile = Ti.Filesystem.getFile(oldpath, file)
  if oldfile.exists()
    wd.info "Copying #{oldpath} / #{file}..."
    current_path = Ti.Filesystem.applicationDataDirectory
    for folder in path
      dir = Ti.Filesystem.getFile(current_path, folder)
      unless dir.exists()
        wd.info "Creating directory #{current_path} / #{folder}..."
        dir.createDirectory()
      else
        wd.info "Directory exists: #{current_path}"      
      current_path = current_path + Ti.Filesystem.separator + folder

    newpath = Ti.Filesystem.applicationDataDirectory + path.join(Ti.Filesystem.separator)
    newfile = Ti.Filesystem.getFile(newpath, file)
    newfile.write(oldfile.read())
    wd.info "Wrote to #{newpath} / #{file}."
  else
    wd.debug "File #{oldpath} / #{file} doesn't exist!"
  oldfile = null
  newfile = null


copyWebViews = ->
  copyFile ["html"], "tale.html"
  copyFile ["html"], "style.css"
  copyFile ["html", "libs"], "handlebars-1.0.0.beta.6.js"
  copyFile ["html", "libs"], "underscore.js"
  copyFile ["html", "libs"], "zepto.min.js"
  copyFile ["html", "js"], "loadPhotos.js"


# ========================================================================= #
#   LOCAL DB METHODS                                                        #
# ========================================================================= #

wd.db.get = (sql, args = []) ->
  rows = []
  db = Ti.Database.open(wd.db_name)
  result = db.execute(sql, args)
  while result.isValidRow()
    row = {}
    col = 0
    fieldCount = if wd.android then result.fieldCount else result.fieldCount()
    while col < fieldCount  # Android bug
      row[result.fieldName(col)] = result.field(col)
      col += 1
    rows.push row
    result.next()
  result.close()
  db.close()
  return rows
    

# ========================================================================= #
#   CALL                                                                    #
# ========================================================================= #

# wipeDirectory()
bootstrap()
resetProperties()
copyWebViews()

wd.info "userID: #{wd.getint("userID")}"
wd.info "talesLastPull: #{wd.getint("talesLastPull")}"
wd.info "currentTale: #{wd.getint("currentTale")}"
wd.info "nextPhotoID: #{wd.getint("nextPhotoID")}"
