# ========================================================================= #
#   MODEL                                                                   #
# ========================================================================= #

class Model

  @attributes = []

  @configure: (name) ->
    @className = name
    @db_table = name.toLowerCase() + "s"
    this

  @attr_accessible: (attributes...) ->
    @attributes = attributes
    this
        
  @create: (args = {}) ->
    obj = new this
    for key, val of args
      obj[key] = val
    if obj.save()
      return obj
    return false
    
    
  # ========================================================================= #
  #   INSTANCE METHODS                                                        #
  # ========================================================================= #

  validate: ->

  validates: ->
    error = @validate()
    unless error
      return true
    alert error
    return false
  
  save: ->
    if @validates()
      obj = Steel.filter(this, @constructor.attributes)
      keys = _.keys(obj).join(", ")
      values = _.values(obj)
      marks = _.map(values, -> "?").join(", ")
      db = if wd.db.tsOpen then wd.db.ts else Ti.Database.open(wd.db_name)
      db.execute "INSERT INTO #{@constructor.db_table} (#{keys}) VALUES (#{marks})", values
      db.close() unless wd.db.tsOpen
      return true
    return false
      
  update: (args = {}) ->
    for key, val of args
      @[key] = val
    if @validates()
      frag = _.chain(args).keys().map((k) -> k + " = ?").value().join(", ")
      params = _.values(args)
      params.push @id
      db = if wd.db.tsOpen then wd.db.ts else Ti.Database.open(wd.db_name)
      db.execute "UPDATE #{@constructor.db_table} SET #{frag} WHERE id = ?", params
      db.close() unless wd.db.tsOpen
      return true
    return false
  
  destroy: ->
    db = Ti.Database.open(wd.db_name)
    db.execute "DELETE FROM #{@constructor.db_table} WHERE id = ?", @id
    db.close()


  # ========================================================================= #
  #   SELECT METHODS                                                          #
  # ========================================================================= #

  @find: (id) ->
    rows = wd.db.get "SELECT * FROM #{@db_table} WHERE id = ?", [id]
    wd.Steel.objectify(rows, this)

  @first: ->
    rows = wd.db.get "SELECT * FROM #{@db_table} ORDER BY id ASC LIMIT 1"
    wd.Steel.objectify(rows, this)

  @last: ->
    rows = wd.db.get "SELECT * FROM #{@db_table} ORDER BY id DESC LIMIT 1"
    wd.Steel.objectify(rows, this)

  @count: ->
    db = Ti.Database.open(wd.db_name)
    result = db.execute("SELECT COUNT(*) FROM #{@db_table}")
    count = result.field(0)
    result.close()
    db.close()
    return count

  @all: ->
    rows = wd.db.get "SELECT * FROM #{@db_table}"
    wd.Steel.objectify_many(rows, this)
  
  @where: (where_frag, args = []) ->
    rows = wd.db.get "SELECT * FROM #{@db_table} WHERE #{where_frag}", args
    wd.Steel.objectify_many(rows, this)
  
# ========================================================================= #
#   CONTROLLER                                                              #
# ========================================================================= #
# 
# class Controller
# 

# ========================================================================= #
#   HELPERS                                                                 #
# ========================================================================= #

Steel = {}

Steel.filter = (obj, attrs) ->
  new_obj = {}
  for attr in attrs
    if obj.hasOwnProperty attr
      new_obj[attr] = obj[attr]
  return new_obj
  
Steel.objectify = (rows, model) ->
  if _.isEmpty(rows)
    return false
  else
    obj = new model
    for key of rows[0]
      obj[key] = rows[0][key]
    return obj

Steel.objectify_many = (rows, model) ->
  objs = []
  unless _.isEmpty(rows)
    for row in rows
      obj = new model
      for key of row
        obj[key] = row[key]
      objs.push obj
  return objs

  
# ========================================================================= #
#   GLOBALS                                                                 #
# ========================================================================= #

wd.Steel            = Steel
# wd.Steel.Controller = Controller
wd.Steel.Model      = Model

