(function() {
  var Photos;

  Photos = (function() {

    function Photos() {}

    Photos.prototype.index = function() {
      this.gallery();
      return wd.ui.showTab("photos");
    };

    Photos.prototype.camera = function() {
      var _this = this;
      return Ti.Media.showCamera({
        saveToPhotoGallery: true,
        mediaTypes: [Ti.Media.MEDIA_TYPE_PHOTO],
        animated: false,
        success: function(e) {
          return _this.create({
            media: _this.process(e.media),
            _s: function(photo) {
              wd.photos.push();
              return wd.ui.showTab("home");
            }
          });
        },
        cancel: function() {
          return wd.ui.showTab("home");
        },
        error: function(error) {
          if (error.code === Ti.Media.NO_CAMERA) {
            wd.info("No camera detected");
            return _this.gallery();
          } else {
            return alert("Camera error: " + error.code);
          }
        }
      });
    };

    Photos.prototype.gallery = function() {
      var _this = this;
      wd.info("Opening gallery");
      return Ti.Media.openPhotoGallery({
        success: function(e) {
          if (e.mediaType === Ti.Media.MEDIA_TYPE_PHOTO) {
            return _this.create({
              media: _this.process(e.media),
              _s: function(photo) {
                wd.photos.push();
                return wd.ui.showTab("home");
              }
            });
          } else {
            return alert("Please select a photo");
          }
        },
        cancel: function() {
          return wd.ui.showTab("home");
        }
      });
    };

    Photos.prototype.create = function(args) {
      var f, filename, photo, photoID, taleID;
      photoID = wd.getint("nextPhotoID");
      taleID = wd.getint("currentTale");
      filename = "p" + (wd.rjust(photoID, 6)) + ".jpg";
      photo = wd.Photo.create(_.extend(args, {
        filename: filename,
        tale_id: taleID,
        remote_tale_id: wd.Tale.find(taleID).remote_id
      }));
      wd.info("Writing photo " + filename);
      if (photo) {
        wd.setint("nextPhotoID", photoID + 1);
        f = Ti.Filesystem.getFile(Ti.Filesystem.applicationDataDirectory, filename);
        f.write(args.media);
        f = null;
        wd.info("Photo " + filename + " written");
        this.addNewPhoto(photo);
        if (args._s) return args._s();
      }
    };

    Photos.prototype.addNewPhoto = function(photo) {
      if (wd.getint("currentTale") === photo.tale_id) {
        wd.info("Loading new photo " + photo.filename + " to homeWebView...");
        wd.ui.homeWebView.evalJS("wd.addNewPhoto('" + photo.filename + "')");
      }
      if (wd.ui.taleShowWin && wd.shownTale && wd.shownTale.id === photo.tale_id) {
        wd.info("Loading new photo " + photo.filename + " to taleWebView...");
        return wd.ui.taleWebView.evalJS("wd.addNewPhoto('" + photo.filename + "')");
      }
    };

    Photos.prototype.pull = function(tale) {
      var last_pull,
        _this = this;
      wd.info("Pulling photos for " + tale.remote_id);
      last_pull = tale.last_pull;
      tale.update({
        last_pull: Math.round(+new Date() / 1000)
      });
      return wd.net.get("api/photos/pull.json?since=" + last_pull + "&tale=" + tale.remote_id, {
        _s: function(xhr) {
          var params, photo_remote_ids, row, _i, _len, _ref;
          wd.info("Pulled " + (_.size(JSON.parse(xhr.responseText))) + " photos for " + tale.remote_id);
          photo_remote_ids = _(wd.Photo.all()).pluck("remote_id");
          wd.db.ts = Ti.Database.open(wd.db_name);
          wd.db.tsOpen = true;
          wd.db.ts.execute("BEGIN IMMEDIATE TRANSACTION");
          _ref = JSON.parse(xhr.responseText);
          for (_i = 0, _len = _ref.length; _i < _len; _i++) {
            row = _ref[_i];
            if (!_(photo_remote_ids).include(row.id)) {
              wd.info("Getting photo with remote_id " + row.id);
              params = {
                remote_id: row.id,
                remote_tale_id: row.tale_id,
                user_id: row.user_id,
                created_at: row.created_at,
                updated_at: row.updated_at,
                synced: 1
              };
              wd.net.get(row.image.url.slice(1), {
                _s: function(xhr, params) {
                  wd.info("Creating photo with remote_id " + params.remote_id);
                  return _this.create(_.extend(params, {
                    media: xhr.responseData
                  }));
                },
                _f: function(xhr) {
                  return wd.info("Error getting image: " + xhr.responseText);
                }
              }, params);
            }
          }
          wd.db.ts.execute("COMMIT TRANSACTION");
          wd.db.tsOpen = false;
          return wd.db.ts.close();
        },
        _f: function() {
          return wd.debug("Error pulling photos!");
        }
      });
    };

    Photos.prototype.push = function() {
      var f, media, params, photo, unsynced_photos, _i, _len, _results;
      unsynced_photos = _.map(wd.Photo.where("synced = ?", 0), function(photo) {
        return wd.Steel.filter(photo, ["id", "filename", "remote_tale_id", "created_at", "updated_at"]);
      });
      wd.info("Pushing " + (_.size(unsynced_photos)) + " photos");
      _results = [];
      for (_i = 0, _len = unsynced_photos.length; _i < _len; _i++) {
        photo = unsynced_photos[_i];
        f = Ti.Filesystem.getFile(Ti.Filesystem.applicationDataDirectory, photo.filename);
        media = f.read();
        params = {
          "photo[id]": photo.id,
          "photo[tale_id]": photo.remote_tale_id,
          "photo[created_at]": photo.created_at,
          "photo[updated_at]": photo.updated_at,
          "photo[image]": media,
          "user[id]": wd.getint("userID")
        };
        _results.push(wd.net.post("api/photos/push.json", params, {
          _s: function(xhr) {
            var errorMsg, response;
            response = JSON.parse(xhr.responseText);
            if (_.has(response, "remote_id")) {
              return wd.Photo.find(response.id).update({
                synced: 1,
                remote_id: response.remote_id
              });
            } else {
              errorMsg = response.errors.join(", ");
              return alert(errorMsg);
            }
          },
          _f: function() {
            return wd.debug("Error pushing photos!");
          },
          onsendstream: function(e) {
            return wd.info("send stream: " + e.progress);
          }
        }, media = null));
      }
      return _results;
    };

    Photos.prototype.process = function(media) {
      return this.resize(this.crop(media));
    };

    Photos.prototype.crop = function(media) {
      var cropped, height, img, length, postImg, preImg, width;
      if (wd.iphone) {
        width = media.width;
        height = media.height;
      } else {
        preImg = Ti.UI.createImageView({
          image: media
        });
        width = preImg.toBlob().width;
        height = preImg.toBlob().height;
      }
      img = Ti.UI.createImageView({
        image: media,
        width: width,
        height: height
      });
      length = height < width ? height : width;
      cropped = Ti.UI.createView({
        top: 0,
        left: 0,
        width: length,
        height: length
      });
      cropped.add(img);
      img.left = -(width - length) / 2;
      img.top = -(height - length) / 2;
      if (wd.android) {
        postImg = Ti.UI.createImageView({
          image: cropped.toImage()
        });
        return postImg.toBlob();
      }
      return cropped.toImage();
    };

    Photos.prototype.resize = function(media) {
      var resized;
      resized = Ti.UI.createImageView({
        image: media,
        height: 320,
        width: 320
      });
      if (wd.iphone) {
        return resized.toImage();
      } else {
        return resized.toBlob();
      }
    };

    return Photos;

  })();

  wd.photos = new Photos;

}).call(this);
