(function() {
  var bootstrap, copyFile, copyWebViews, resetProperties, wipeDirectory;

  wd.db = {};

  bootstrap = function() {
    var db;
    db = Ti.Database.open(wd.db_name);
    db.execute("DROP TABLE IF EXISTS tales");
    db.execute("DROP TABLE IF EXISTS collabs");
    db.execute("DROP TABLE IF EXISTS photos");
    db.execute("CREATE TABLE IF NOT EXISTS tales (                id INTEGER PRIMARY KEY,                remote_id INTEGER,                title TEXT,                synced INTEGER DEFAULT 0,                last_pull INTEGER,                created_at DATETIME DEFAULT CURRENT_TIMESTAMP,                updated_at DATETIME DEFAULT CURRENT_TIMESTAMP              )");
    db.execute("CREATE TABLE IF NOT EXISTS collabs (                id INTEGER PRIMARY KEY,                remote_id INTEGER,                collab_tale_id INTEGER,                collaborator_id INTEGER,                created_at DATETIME DEFAULT CURRENT_TIMESTAMP,                updated_at DATETIME DEFAULT CURRENT_TIMESTAMP              )");
    db.execute("CREATE TABLE IF NOT EXISTS photos (                id INTEGER PRIMARY KEY,                remote_id INTEGER,                synced INTEGER DEFAULT 0,                tale_id INTEGER,                user_id INTEGER,                remote_tale_id INTEGER,                filename TEXT,                url TEXT,                created_at DATETIME DEFAULT CURRENT_TIMESTAMP,                updated_at DATETIME DEFAULT CURRENT_TIMESTAMP              )");
    return db.close();
  };

  resetProperties = function() {
    wd.clear("talesLastPull");
    wd.setint("currentTale", 1);
    return wd.setint("nextPhotoID", 1);
  };

  wipeDirectory = function() {
    var f, filename, i, no_file, _results;
    no_file = false;
    i = 1;
    _results = [];
    while (!no_file) {
      filename = "p" + (wd.rjust(i, 6)) + ".jpg";
      f = Ti.Filesystem.getFile(Ti.Filesystem.applicationDataDirectory, filename);
      if (f.exists()) {
        f.deleteFile();
        _results.push(i += 1);
      } else {
        _results.push(no_file = true);
      }
    }
    return _results;
  };

  copyFile = function(path, file) {
    var current_path, dir, folder, newfile, newpath, oldfile, oldpath, _i, _len;
    oldpath = Ti.Filesystem.resourcesDirectory + path.join(Ti.Filesystem.separator);
    oldfile = Ti.Filesystem.getFile(oldpath, file);
    if (oldfile.exists()) {
      wd.info("Copying " + oldpath + " / " + file + "...");
      current_path = Ti.Filesystem.applicationDataDirectory;
      for (_i = 0, _len = path.length; _i < _len; _i++) {
        folder = path[_i];
        dir = Ti.Filesystem.getFile(current_path, folder);
        if (!dir.exists()) {
          wd.info("Creating directory " + current_path + " / " + folder + "...");
          dir.createDirectory();
        } else {
          wd.info("Directory exists: " + current_path);
        }
        current_path = current_path + Ti.Filesystem.separator + folder;
      }
      newpath = Ti.Filesystem.applicationDataDirectory + path.join(Ti.Filesystem.separator);
      newfile = Ti.Filesystem.getFile(newpath, file);
      newfile.write(oldfile.read());
      wd.info("Wrote to " + newpath + " / " + file + ".");
    } else {
      wd.debug("File " + oldpath + " / " + file + " doesn't exist!");
    }
    oldfile = null;
    return newfile = null;
  };

  copyWebViews = function() {
    copyFile(["html"], "tale.html");
    copyFile(["html"], "style.css");
    copyFile(["html", "libs"], "handlebars-1.0.0.beta.6.js");
    copyFile(["html", "libs"], "underscore.js");
    copyFile(["html", "libs"], "zepto.min.js");
    return copyFile(["html", "js"], "loadPhotos.js");
  };

  wd.db.get = function(sql, args) {
    var col, db, fieldCount, result, row, rows;
    if (args == null) args = [];
    rows = [];
    db = Ti.Database.open(wd.db_name);
    result = db.execute(sql, args);
    while (result.isValidRow()) {
      row = {};
      col = 0;
      fieldCount = wd.android ? result.fieldCount : result.fieldCount();
      while (col < fieldCount) {
        row[result.fieldName(col)] = result.field(col);
        col += 1;
      }
      rows.push(row);
      result.next();
    }
    result.close();
    db.close();
    return rows;
  };

  bootstrap();

  resetProperties();

  copyWebViews();

  wd.info("userID: " + (wd.getint("userID")));

  wd.info("talesLastPull: " + (wd.getint("talesLastPull")));

  wd.info("currentTale: " + (wd.getint("currentTale")));

  wd.info("nextPhotoID: " + (wd.getint("nextPhotoID")));

}).call(this);
