(function() {

  wd.ActiveRecord = (function() {
    var validates;

    function ActiveRecord() {}

    ActiveRecord.configure = function(name) {
      this.className = name;
      this.db_table = name.toLowerCase() + "s";
      return this;
    };

    ActiveRecord.attr_accessible = function(attributes) {
      this.attributes = attributes;
      return this;
    };

    ActiveRecord.prototype.validate = function() {};

    validates = function() {
      var error;
      return error = this.validate();
    };

    ActiveRecord.prototype.save = function() {
      var attr, attr_acc, db, keys, marks, values;
      if (this.validates()) {
        attr_acc = this.model.attr_accessible;
        attr = _.filter_obj(this, function(v, k) {
          return _.include(attr_acc, k);
        });
        keys = _.keys(attr).join(", ");
        values = _.values(attr);
        marks = _.map(values, function() {
          return "?";
        }).join(", ");
        db = wd.db.tsOpen ? wd.db.ts : Ti.Database.open(wd.db_name);
        db.execute("INSERT INTO " + this.db_table + " (" + keys + ") VALUES (" + marks + ")", values);
        if (!wd.db.tsOpen) db.close();
        return true;
      } else {
        return false;
      }
    };

    ActiveRecord.prototype.update = function(args) {
      var db, frag, params;
      if (args == null) args = {};
      frag = _.chain(args).keys().map(function(k) {
        return k + " = ?";
      }).value().join(", ");
      params = _.values(args);
      params.push(this.id);
      db = Ti.Database.open(wd.db_name);
      db.execute("UPDATE " + this.db_table + " SET " + frag + " WHERE id = ?", params);
      return db.close();
    };

    ActiveRecord.prototype.destroy = function() {
      var db;
      db = Ti.Database.open(wd.db_name);
      db.execute("DELETE FROM " + this.db_table + " WHERE id = ?", this.id);
      return db.close();
    };

    ActiveRecord.find = function(id) {
      var rows;
      rows = wd.db.get("SELECT * FROM " + this.db_table + " WHERE id = ?", [id]);
      return this.objectify(rows);
    };

    ActiveRecord.first = function() {
      var rows;
      rows = wd.db.get("SELECT * FROM " + this.db_table + " ORDER BY id ASC LIMIT 1");
      return this.objectify(rows);
    };

    ActiveRecord.last = function() {
      var rows;
      rows = wd.db.get("SELECT * FROM " + this.db_table + " ORDER BY id DESC LIMIT 1");
      return this.objectify(rows);
    };

    ActiveRecord.count = function() {
      var count, db, result;
      db = Ti.Database.open(wd.db_name);
      result = db.execute("SELECT COUNT(*) FROM " + this.db_table);
      count = result.field(0);
      result.close();
      db.close();
      return count;
    };

    ActiveRecord.all = function() {
      var rows;
      rows = wd.db.get("SELECT * FROM " + this.db_table);
      return this.objectify_many(rows);
    };

    ActiveRecord.where = function(where_frag, args) {
      var rows;
      if (args == null) args = [];
      rows = wd.db.get("SELECT * FROM " + this.db_table + " WHERE " + where_frag, args);
      return this.objectify_many(rows);
    };

    ActiveRecord.objectify = function(rows) {
      var key, obj;
      if (_.isEmpty(rows)) {
        return false;
      } else {
        obj = new this;
        for (key in rows[0]) {
          obj[key] = rows[0][key];
        }
        return obj;
      }
    };

    ActiveRecord.objectify_many = function(rows) {
      var key, obj, objs, row, _i, _len;
      objs = [];
      if (!_.isEmpty(rows)) {
        for (_i = 0, _len = rows.length; _i < _len; _i++) {
          row = rows[_i];
          obj = new this;
          for (key in row) {
            obj[key] = row[key];
          }
          objs.push(obj);
        }
      }
      return objs;
    };

    return ActiveRecord;

  })();

}).call(this);
