(function() {
  var Model, Steel,
    __slice = Array.prototype.slice;

  Model = (function() {

    function Model() {}

    Model.attributes = [];

    Model.configure = function(name) {
      this.className = name;
      this.db_table = name.toLowerCase() + "s";
      return this;
    };

    Model.attr_accessible = function() {
      var attributes;
      attributes = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      this.attributes = attributes;
      return this;
    };

    Model.create = function(args) {
      var key, obj, val;
      if (args == null) args = {};
      obj = new this;
      for (key in args) {
        val = args[key];
        obj[key] = val;
      }
      if (obj.save()) return obj;
      return false;
    };

    Model.prototype.validate = function() {};

    Model.prototype.validates = function() {
      var error;
      error = this.validate();
      if (!error) return true;
      alert(error);
      return false;
    };

    Model.prototype.save = function() {
      var db, keys, marks, obj, values;
      if (this.validates()) {
        obj = Steel.filter(this, this.constructor.attributes);
        keys = _.keys(obj).join(", ");
        values = _.values(obj);
        marks = _.map(values, function() {
          return "?";
        }).join(", ");
        db = wd.db.tsOpen ? wd.db.ts : Ti.Database.open(wd.db_name);
        db.execute("INSERT INTO " + this.constructor.db_table + " (" + keys + ") VALUES (" + marks + ")", values);
        if (!wd.db.tsOpen) db.close();
        return true;
      }
      return false;
    };

    Model.prototype.update = function(args) {
      var db, frag, key, params, val;
      if (args == null) args = {};
      for (key in args) {
        val = args[key];
        this[key] = val;
      }
      if (this.validates()) {
        frag = _.chain(args).keys().map(function(k) {
          return k + " = ?";
        }).value().join(", ");
        params = _.values(args);
        params.push(this.id);
        db = wd.db.tsOpen ? wd.db.ts : Ti.Database.open(wd.db_name);
        db.execute("UPDATE " + this.constructor.db_table + " SET " + frag + " WHERE id = ?", params);
        if (!wd.db.tsOpen) db.close();
        return true;
      }
      return false;
    };

    Model.prototype.destroy = function() {
      var db;
      db = Ti.Database.open(wd.db_name);
      db.execute("DELETE FROM " + this.constructor.db_table + " WHERE id = ?", this.id);
      return db.close();
    };

    Model.find = function(id) {
      var rows;
      rows = wd.db.get("SELECT * FROM " + this.db_table + " WHERE id = ?", [id]);
      return wd.Steel.objectify(rows, this);
    };

    Model.first = function() {
      var rows;
      rows = wd.db.get("SELECT * FROM " + this.db_table + " ORDER BY id ASC LIMIT 1");
      return wd.Steel.objectify(rows, this);
    };

    Model.last = function() {
      var rows;
      rows = wd.db.get("SELECT * FROM " + this.db_table + " ORDER BY id DESC LIMIT 1");
      return wd.Steel.objectify(rows, this);
    };

    Model.count = function() {
      var count, db, result;
      db = Ti.Database.open(wd.db_name);
      result = db.execute("SELECT COUNT(*) FROM " + this.db_table);
      count = result.field(0);
      result.close();
      db.close();
      return count;
    };

    Model.all = function() {
      var rows;
      rows = wd.db.get("SELECT * FROM " + this.db_table);
      return wd.Steel.objectify_many(rows, this);
    };

    Model.where = function(where_frag, args) {
      var rows;
      if (args == null) args = [];
      rows = wd.db.get("SELECT * FROM " + this.db_table + " WHERE " + where_frag, args);
      return wd.Steel.objectify_many(rows, this);
    };

    return Model;

  })();

  Steel = {};

  Steel.filter = function(obj, attrs) {
    var attr, new_obj, _i, _len;
    new_obj = {};
    for (_i = 0, _len = attrs.length; _i < _len; _i++) {
      attr = attrs[_i];
      if (obj.hasOwnProperty(attr)) new_obj[attr] = obj[attr];
    }
    return new_obj;
  };

  Steel.objectify = function(rows, model) {
    var key, obj;
    if (_.isEmpty(rows)) {
      return false;
    } else {
      obj = new model;
      for (key in rows[0]) {
        obj[key] = rows[0][key];
      }
      return obj;
    }
  };

  Steel.objectify_many = function(rows, model) {
    var key, obj, objs, row, _i, _len;
    objs = [];
    if (!_.isEmpty(rows)) {
      for (_i = 0, _len = rows.length; _i < _len; _i++) {
        row = rows[_i];
        obj = new model;
        for (key in row) {
          obj[key] = row[key];
        }
        objs.push(obj);
      }
    }
    return objs;
  };

  wd.Steel = Steel;

  wd.Steel.Model = Model;

}).call(this);
